clear all;

%Specs of the simulation---------------------------------------------------
j=8;                                                                       %Parameter determining time step and mesh size. (j=8 is quick, j=9 takes a little, j=10 takes LONG)

t = 10;                                                                    %Time
L = 2^j;                                                                   %Number of spatial grid points
tau = 2^(-j);                                                              %Time step size
N = t*2^j;                                                                 %Number of time steps
p = 2;                                                                     %L^p-norm to compute
 
l = 20;                                                                    %Length of spatial interval
h = l/(L+1);                                                               %spatial mesh size
xi = 1/l*[0:L/2, -L/2+1:-1]';                                              %Fourier modes with Matlab ordering
x = linspace(0,l-h,L)';                                                    %space grid

speed = 8;                                                                 %Determines the speed at which the animation plays

%Specs of the wavetrain----------------------------------------------------
mu=10;                                                                     %temporal frequency 
q=2*pi;                                                                    %wavenumber
r = log(mu^2-q^2-1)/2;

%Setting up initial values-------------------------------------------------
p0= x.*0+4*(1+i)*exp(-25*(x-10).^2)*1;                                     %initial perturbation of u0
pt0= x.*0+40*(1+i)*exp(-25*(x-10).^2)*1;                                   %initial perturbation of ut0

u0 = exp(r+1i*x.*q)+ p0;                                                   %Initial values
ut0 = 1i*mu*exp(r+1i*x.*q)+pt0;

%Plot initial perturbation-------------------------------------------------
set(gcf, 'Position', get(0, 'Screensize'));
sgtitle('Initial Conditions')

subplot(2,2,1);                           
plot(x, real( exp(r+1i*x.*q)),x,real(p0));
title('Real part: Unperturbed position (blue) and perturbation (red)');
xlabel('x')
ylim([-1.5*exp(r) 1.5*exp(r)]);

subplot(2,2,2);
plot(x, imag( exp(r+1i*x.*q)),x,imag(p0));
title('Imaginary part: Unperturbed position (blue) and perturbation (red)');
xlabel('x')
ylim([-1.5*exp(r) 1.5*exp(r)]);

subplot(2,2,3);
plot(x, real( 1i*mu*exp(r+1i*x.*q)),x,real(pt0));
title('Real part: Unperturbed velocity (blue) and perturbation (red)');
xlabel('x')
ylim([-1.5*mu*exp(r) 1.5*mu*exp(r)]);

subplot(2,2,4);
plot(x, imag( 1i*mu*exp(r+1i*x.*q)),x,imag(pt0));
title('Imaginary part: Unperturbed velocity (blue) and perturbation (red)');
xlabel('x')
ylim([-1.5*mu*exp(r) 1.5*mu*exp(r)]);

figure;

%Compute the solution------------------------------------------------------
U = Strang_Klein_Gordon(u0,ut0,tau,xi,N);                                   %Compute solution with Strang splitting


%Compute polar coordinates-------------------------------------------------
W = U;
rho = U;
dxrho = U;
theta = U;
dxtheta = U;
for j=1:N                                                                  %Compute rho, theta & derivatives
W(:,j) = U(:,j)./(exp(1i*x.*q+1i*j*tau*mu));
rho(:,j) = real(log(W(:,j)./exp(r)));
theta(:,j) = imag(log(W(:,j)./exp(r)));                                    %WARNING: Defining theta this way does not preclude jumps of 2*Pi (Fine for small initial perturbations). 

for i=1:L-1
dxrho(i,j)=(rho(i+1,j)-rho(i,j))./h;
dxtheta(i,j)=(theta(i+1,j)-theta(i,j))./h;
end
dxrho(L,j)=(rho(L,j)-rho(L-1,j))./h;
dxtheta(L,j)=(theta(L,j)-theta(L-1,j))./h;
end

%Compute norms of polar coordinates----------------------------------------
L2_rho = zeros(N,1);
Lp_theta = zeros(N,1);
L2_dxrho = zeros(N,1);
L2_dxtheta = zeros(N,1);
for j=1:N                                                                  %Compute L^2-norms
L2_rho(j) = trapz(x,rho(:,j).^2)^(1/2);
L2_dxrho(j) = trapz(x,dxrho(:,j).^2)^(1/2);
L2_dxtheta(j) = trapz(x,dxtheta(:,j).^2)^(1/2);
end
if p == Inf                                                                %Compute L^p-norm for theta
    for j=1:N                                                               
        Lp_theta(j) = max(abs(theta(:,j)));
    end
else
    for j=1:N                                                               
        Lp_theta(j) = trapz(x,theta(:,j).^p)^(1/p);
    end
end
%Plot norm growth (Fig. 1)-------------------------------------------------
set(gcf, 'Position', get(0, 'Screensize'));
sgtitle('Norm growth');

subplot(2,2,1);
y = linspace(0,N*tau-N*tau/(N+1),N)';
plot(y,L2_rho);
ylabel('||\rho(t)||_2');
xlabel('t');

subplot(2,2,3);
y = linspace(0,N*tau-N*tau/(N+1),N)';
plot(y,L2_dxrho);
ylabel('||\rho_x(t)||_2');
xlabel('t');

subplot(2,2,4);
y = linspace(0,N*tau-N*tau/(N+1),N)';
plot(y,L2_dxtheta);
ylabel('||\theta_x(t)||_2');
xlabel('t');

subplot(2,2,2);
y = linspace(0,N*tau-N*tau/(N+1),N)';
plot(y,Lp_theta);
if p == Inf                                                                
    ylabel('||\theta(t)||_\infty');
else
    ylabel(append('||\theta(t)||_',int2str(p)));
end
xlabel('t');

set(gcf,'Units','inches');
screenposition = get(gcf,'Position');
set(gcf,...
    'PaperPosition',[0 0 screenposition(3:4)],...
    'PaperSize',[screenposition(3:4)]);
print -dpdf -vector Fig1

figure;

%Loglog plot for theta (Fig. 2)--------------------------------------------

y = linspace(3*N*tau/4,N*tau-N*tau/(N+1),N/4)';                            %Only take last quarter of data points into account
Lp_theta_2 = linspace(3*N*tau/4,N*tau-N*tau/(N+1),N/4)';
for j=1:N/4
    Lp_theta_2(j) = Lp_theta(j+3*N/4);
end
modelfun = @(b,x) b(1)+b(2)*x(:);                                          %Fit linear function to Loglog plot
approx = nlinfit(reallog(y),reallog(Lp_theta_2),modelfun,[0 1]);
y = linspace(0,N*tau-N*tau/(N+1),N)';
X = ['New best fit for L^',num2str(p),'-norm growth of theta ', 'x^', num2str(approx(2))];
disp(X);

set(gcf, 'Position', get(0, 'Screensize'));                                %Loglog plot
sgtitle('Loglog plot');

subplot(1,1,1);                                                            
plot(reallog(y),reallog(Lp_theta),reallog(y),approx(1)+approx(2)*reallog(y));
if p == Inf                                                                
    ylabel('log(||\theta(t)||_\infty)');
else
    ylabel(append('log(||\theta||_',int2str(p),')'));
end
xlabel('log(t)');

set(gcf,'Units','inches');
screenposition = get(gcf,'Position');
set(gcf,...
    'PaperPosition',[0 0 screenposition(3:4)],...
    'PaperSize',[screenposition(3:4)]);
print -dpdf -vector Fig2

figure;

%Plot perturbed solution and theta at different times (Fig. 3)-------------

b = max(reshape(abs(real(U)),1,[]));
b4 = max(reshape(abs(theta),1,[]));

set(gcf, 'Position', get(0, 'Screensize'));                                
sgtitle('Time evolution')

subplot(3,2,1);                                                            %Starting with time t_0=0.
yyaxis left
plot(x,real(U(:,1)),'Color',[0, 0.4470, 0.7410]);
ylim([-1.1*b 1.1*b]);
yyaxis right
plot(x,theta(:,1),'Color',[0.8500, 0.3250, 0.0980]);
ylim([-0.3*b4 5*b4]);
xlabel('t_0=0');

subplot(3,2,2);                                                            %Plot at time t_0=2.
yyaxis left
plot(x,real(U(:,2*N/10)),'Color',[0, 0.4470, 0.7410]);
ylim([-1.1*b 1.1*b]);
yyaxis right
plot(x,theta(:,2*N/10),'Color',[0.8500, 0.3250, 0.0980]);
ylim([-0.3*b4 5*b4]);
xlabel('t_0=2');

subplot(3,2,3);                                                            %Plot at time t_0=4. And so on.
yyaxis left
plot(x,real(U(:,4*N/10)),'Color',[0, 0.4470, 0.7410]);
ylim([-1.1*b 1.1*b]);
yyaxis right
plot(x,theta(:,4*N/10),'Color',[0.8500, 0.3250, 0.0980]);
ylim([-0.3*b4 5*b4]);
xlabel('t_0=4');

subplot(3,2,4);
yyaxis left
plot(x,real(U(:,6*N/10)),'Color',[0, 0.4470, 0.7410]);
ylim([-1.1*b 1.1*b]);
yyaxis right
plot(x,theta(:,6*N/10),'Color',[0.8500, 0.3250, 0.0980]);
ylim([-0.3*b4 5*b4]);
xlabel('t_0=6');

subplot(3,2,5);
yyaxis left
plot(x,real(U(:,8*N/10)),'Color',[0, 0.4470, 0.7410]);
ylim([-1.1*b 1.1*b]);
yyaxis right
plot(x,theta(:,8*N/10),'Color',[0.8500, 0.3250, 0.0980]);
ylim([-0.3*b4 5*b4]);
xlabel('t_0=8');

subplot(3,2,6);
yyaxis left
plot(x,real(U(:,N)),'Color',[0, 0.4470, 0.7410]);
ylim([-1.1*b 1.1*b]);
yyaxis right
plot(x,theta(:,N),'Color',[0.8500, 0.3250, 0.0980]);
ylim([-0.3*b4 5*b4]);
xlabel('t_0=10');

set(gcf,'Units','inches');
screenposition = get(gcf,'Position');
set(gcf,...
    'PaperPosition',[0 0.2 screenposition(3:4)],...
    'PaperSize',[screenposition(3:4)]);
print -dpdf -vector Fig3

figure;


%BONUS: Animating the dynamics---------------------------------------------

set(gcf, 'Position', get(0, 'Screensize'));                                
sgtitle('Time evolution')

subplot(4,2,5);
y = linspace(0,N*tau-N*tau/(N+1),N)';
plot(y,L2_rho);
title('L2-norm of rho');
xlabel('t');

subplot(4,2,6);
y = linspace(0,N*tau-N*tau/(N+1),N)';
plot(y,Lp_theta);
if p == Inf                                                                
    ylabel('||\theta(t)||_\infty');
else
    ylabel(append('||\theta(t)||_',int2str(p)));
end
xlabel('t');

subplot(4,2,7);
y = linspace(0,N*tau-N*tau/(N+1),N)';
plot(y,L2_dxrho);
title('L2-norm of dxrho');
xlabel('t');

subplot(4,2,8);
y = linspace(0,N*tau-N*tau/(N+1),N)';
plot(y,L2_dxtheta);
title('L2-norm of dxtheta');
xlabel('t');

b = max(reshape(abs(U),1,[]));

b3 = max(reshape(abs(rho),1,[]));
b4 = max(reshape(abs(theta),1,[]));


for j1 = 1:N/speed
    j=j1*speed;

    sgtitle(['Time evolution t=',num2str(round(j*tau,2))])

    subplot(4,2,1);
    plot(x,real(U(:,j)),x,real(U(:,j)-exp(r+1i*x.*q+1i*mu*tau*j)));
    title(['Real part of perturbed solution (blue) and perturbation (red)']);
    xlabel('x');
    ylim([-1.5*b 1.5*b]);

    subplot(4,2,2);
    plot(x,imag(U(:,j)),x,imag(U(:,j)-exp(r+1i*x.*q+1i*mu*tau*j)));
    title(['Imaginary part of perturbed solution (blue) and perturbation (red)']);
    xlabel('x');
    ylim([-1.5*b 1.5*b]);

    subplot(4,2,3);
    plot(x,rho(:,j));
    title(['Rho']);
    xlabel('x');
    ylim([-1.5*b3 1.5*b3]);

    subplot(4,2,4);
    plot(x,theta(:,j));
    title(['Theta']);
    xlabel('x');
    ylim([-1.5*b4 1.5*b4]);

    pause(0.005);
end